from jpype import *
import time

def run():
	print 'Thread started'
	try:
		print repr(java.awt.Frame)
		javax.swing.JFrame("Test Frame").setVisible(True)
		shutdownGuiEnvironment()
	except JException, ex :
		print ex


startJVM(getDefaultJVMPath())

setupGuiEnvironment(run)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
from jpype import *
import common

class ObjectWrapperTestCase(common.JPypeTestCase) :
    def testCallOverloads(self) :
        # build the harness   
        h = JPackage("jpype.objectwrapper").Test1()

        o = java.lang.Integer(1)
        assert h.Method1(JObject(o, java.lang.Number)) == 1
        assert h.Method1(o) == 2  
        assert h.Method1(JObject(java.lang.Integer(1), java.lang.Object)) == 3
        assert h.Method1(JString("")) == 4



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************

import _jpype, _jclass, JClassUtil
import operator

def _initialize() :
    _jpype.setProxyClass(JProxy)

class JProxy(object) :
    def __init__(self, intf, dict=None, inst=None) :
        actualIntf = None
        
        if isinstance(intf, str) or isinstance(intf, unicode) :
            actualIntf = [ _jclass.JClass(intf) ]
        elif isinstance(intf, _jclass._JavaClass) :
            actualIntf = [ intf ]
        elif operator.isSequenceType(intf) :
            actualIntf = []
            for i in intf :            
                if isinstance(i, str) or isinstance(i, unicode) :
                    actualIntf.append(_jclass.JClass(i))
                elif isinstance(i, _jclass._JavaClass) :
                    actualIntf.append(i)
                else:
                    raise TypeError, "JProxy requires java interface classes or the names of java interfaces classes"                
        else:
            raise TypeError, "JProxy requires java interface classes or the names of java interfaces classes"
                
        for i in actualIntf :
            if not JClassUtil.isInterface(i) :
                raise TypeError, "JProxy requires java interface classes or the names of java interfaces classes : "+i.__name__                 

        if dict is not None and inst is not None :
            raise RuntimeError, "Specify only one of dict and inst"

        self._dict = dict
        self._inst = inst
        
        self._proxy = _jpype.createProxy(self, actualIntf)
        
    def getCallable(self, name) :
        if self._dict is not None :
            return self._dict[name]
        else :
            return getattr(self._inst, name)
        


#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
from jpype import JPackage, java, JFloat
import common, sys

class NumericTestCase(common.JPypeTestCase) :
    def testMathAbs(self) :
        assert java.lang.Math.abs(-10) == 10 

    def testDoubleConversion(self) :
        f = java.lang.Float.MAX_VALUE * 2
        jpype = JPackage("jpype")
        assert jpype.numeric.NumericTest.doubleIsTwiceMaxFloat(f)
        
    def testDoubleIsProperlyConverted(self) :
        if sys.platform.find("linux") != -1 :
            # double comparison on linux is broken ... Nan == 0.0!!! 
            print java.lang.Double.NaN, " != ", 0.0, " -> ", bool(java.lang.Double.NaN != 0.0), " == -> ", bool(java.lang.Double.NaN == 0.0)
        else :
            assert java.lang.Double.NEGATIVE_INFINITY != 0.0
            assert java.lang.Double.MAX_VALUE != 0.0
            assert java.lang.Double.NaN != 0.0
            assert java.lang.Double.POSITIVE_INFINITY != 0.0
    
    def testNegativeJFloatWrapper(self):
        f = JFloat(-1)    



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#	   http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
import _jclass, _jpype

_PROPERTY_ACCESSOR_PREFIX_LEN = 3

def _initialize() :
	_jclass.registerClassCustomizer(PropertiesCustomizer())

def _extract_accessor_pairs(members):
        """Extract pairs of corresponding property access methods
        (getter and setter) from a Java class's members (attributes).

        If a public method with a property's name exists no pair for
        that property will be extracted.

        Returns a dictionary with the property name as key and a tuple
        of (getter method, setter method) as value. A tuple element
        value might be `None` if only a getter or only a setter
        exists.
        """
        accessor_pairs = {}

        for name, member in members.items():
                if not (len(name) > _PROPERTY_ACCESSOR_PREFIX_LEN \
                                and _is_java_method(member)):
                        continue
                access, rest = ( name[:_PROPERTY_ACCESSOR_PREFIX_LEN],
                                 name[_PROPERTY_ACCESSOR_PREFIX_LEN:] )
                property_name = rest[:1].lower() + rest[1:]
                if property_name in members:
                        if _is_java_method(members[property_name]):
                                continue
                if access == 'get' and member.isBeanAccessor():
                        try:
                                pair = accessor_pairs[property_name]
                                pair[0] = member
                        except KeyError:
                                accessor_pairs[property_name] = [member, None]
                elif access == 'set' and member.isBeanMutator():
                        try:
                                pair = accessor_pairs[property_name]
                                pair[1] = member
                        except KeyError:
                                accessor_pairs[property_name] = [None, member]
        return accessor_pairs

def _is_java_method(attribute):
        return isinstance(attribute, _jpype._JavaMethod)

class PropertiesCustomizer(object) :
	def canCustomize(self, name, jc) :
		return True
		
	def customize(self, class_name, jc, bases, members) :
		accessor_pairs = _extract_accessor_pairs(members)
		for attr_name, (getter, setter) in accessor_pairs.items():
                        if attr_name in members:
                                if not getter:
                                        # add default getter if we
                                        # only have a setter
                                        getter = members[attr_name].fget
                                elif not setter:
                                        # add default setter if we
                                        # only have a getter
                                        setter = members[attr_name].fset
                        members[attr_name] = property(getter, setter)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#	   http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
from jpype import JException, java, JavaException, JProxy, JPackage
import common
import traceback

def throwIOException() :
	raise java.io.IOException.PYEXC("Test throw")

def throwByJavaException() :
	JPackage('jpype').exc.ExceptionTest.throwIOException()

class ExceptionTestCase(common.JPypeTestCase) :
	def testExceptionThrown(self) :
		try :
			self.jpype.exc.ExceptionTest.throwRuntime()
			assert False
		except JavaException, ex :
			print 'Caught a Java exception ...'
			if ex.javaClass() is java.lang.RuntimeException :
				print "Caught the exception", ex.message()
				print ex.stacktrace()
			else:
				assert False
		except Exception, ex:
			print ex.__class__, isinstance(ex, JavaException)
			print ex.__class__.__bases__[0].__bases__[0].__bases__
			print JavaException
			assert False
			
		print 'if here, everything is fine'
			
	def testExceptionByJavaClass(self) :
		try :
			self.jpype.exc.ExceptionTest.throwRuntime()
			assert False
		except JException(java.lang.RuntimeException), ex :
			print "Caught the exception", ex.message(), "->", ex.javaClass()
			print ex.stacktrace()
		except Exception, ex:
			print ex
			assert False
			
#	def testThrowException(self) :
#		d = {"throwIOException" : throwIOException, }
#		p = JProxy(self.jpype.exc.ExceptionThrower, dict=d)
#		
#		assert self.jpype.exc.ExceptionTest.delegateThrow(p)

	def testThrowException3(self) :
		d = {"throwIOException" : throwByJavaException, }
		p = JProxy(self.jpype.exc.ExceptionThrower, dict=d)
		
		assert self.jpype.exc.ExceptionTest.delegateThrow(p)

 	def testExceptionPYEXCName(self) :
 		e = self.jpype.exc.ChildTestException()
                name = "jpype.exc.ChildTestExceptionPyRaisable"
                assert name == e.PYEXC.__name__

 	def testExceptionInstanceof(self) :
 		e = self.jpype.exc.ChildTestException()
                assert isinstance(e, self.jpype.exc.ParentTestException)

 	def testExceptionPYEXCInstanceof(self) :
 		e = self.jpype.exc.ChildTestException()
                assert issubclass(e.PYEXC,
                                  self.jpype.exc.ParentTestException.PYEXC)

        def testThrowChildExceptionFromCatchJExceptionParentClass(self) :
		try :
			self.jpype.exc.ExceptionTest.throwChildTestException()
			assert False
		except JException(self.jpype.exc.ParentTestException), ex :
                        pyexc = self.jpype.exc.ChildTestException.PYEXC
                        assert isinstance(ex, pyexc)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************

import _jpype
import _jclass

def _initialize() :
    _jpype.setWrapperClass(_JWrapper)
    _jpype.setStringWrapperClass(JString)

class _JWrapper(object) :
    def __init__(self, v) :
        if v is not None :
            self._value = _jpype.convertToJValue(self.typeName, v)
        else:
            self._value = None
            
    
class JByte(_JWrapper) :
    typeName = "byte"
    
class JShort(_JWrapper) :
    typeName = "short"
    
class JInt(_JWrapper) :
    typeName = "int"

class JLong(_JWrapper) :
    typeName = "long"

class JFloat(_JWrapper) :
    typeName = "float"

class JDouble(_JWrapper) :
    typeName = "double"

class JChar(_JWrapper) :
    typeName = "char"

class JBoolean(_JWrapper) :
    typeName = "boolean"

class JString(_JWrapper) :
    typeName = "java.lang.String"
    
def _getDefaultTypeName(obj) :
    if obj is True or obj is False :
        return 'java.lang.Boolean'
        
    if isinstance(obj, str) or isinstance(obj, unicode) :
        return "java.lang.String"

    if isinstance(obj, int) :
        return "java.lang.Integer"
        
    if isinstance(obj, long) :
        return "java.lang.Long"
        
    if isinstance(obj, float) :
        return "java.lang.Double"

    if isinstance(obj, _jclass._JavaClass) :
        return obj.__javaclassname__
        
    if isinstance(obj, _jclass.java.lang.Class) :
        return obj.__class__.__javaclass__.getName()

    if instance(obj, _JWrapper) :
        return obj.typeName


    raise RuntimeException, "Unable to determine the default type of "+obj.__class__

class JObject(_JWrapper) :
    def __init__(self, v, tp = None) :
        if tp is None :
            tp = _getDefaultTypeName(v)
        if isinstance(tp, _jclass._JavaClass) :
            tp = tp.__javaclass__.getName()
            
        self.typeName = tp
        self._value = _jpype.convertToJValue(tp, v)


#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
import operator
import _jclass

def _initialize() :
    _jclass.registerClassCustomizer(CollectionCustomizer())
    _jclass.registerClassCustomizer(ListCustomizer())
    _jclass.registerClassCustomizer(MapCustomizer())
    _jclass.registerClassCustomizer(IteratorCustomizer())
    _jclass.registerClassCustomizer(EnumerationCustomizer())

def isPythonSequence(v):       
    if operator.isSequenceType(v) :
        if not hasattr(v.__class__, '__metaclass__') or v.__class__.__metaclass__ is _jclass._JavaClass :
            return True
    return False

def _colLength(self) :
    return self.size()
    
def _colIter(self) :    
    return self.iterator()

def _colDelItem(self, i) :
    return self.remove(i)

def _colAddAll(self, v):
    if isPythonSequence(v) :
        r = False
        for i in v :
            r = self.add(i) or r
        return r
    else:
        return self._addAll(v)

def _colRemoveAll(self, v):
    if isPythonSequence(v) :
        r = False
        for i in v :
            r = self.remove(i) or r
        return r
    else:
        return self._removeAll(v)

def _colRetainAll(self, v):
    if isPythonSequence(v) :
        r = _jclass.JClass("java.util.ArrayList")(len(v))
        for i in v :
            r.add(i)
    else:
        r = v
                
    return self._retainAll(r)

class CollectionCustomizer(object) :
    _METHODS = {
        '__len__' : _colLength,
        '__iter__' : _colIter,
        '__delitem__' : _colDelItem,
    }
    
    def canCustomize(self, name, jc) :
        if name == 'java.util.Collection' :
            return True
        return jc.isSubclass('java.util.Collection')
        
    def customize(self, name, jc, bases, members) :
        if name == 'java.util.Collection' :
            members.update(CollectionCustomizer._METHODS)
        else:
            # AddAll is handled by List  
            if (not jc.isSubclass("java.util.List")) and 'addAll' in members :
                members['_addAll'] = members['addAll']
                members['addAll'] = _colAddAll
            if 'removeAll' in members :
                members['_removeAll'] = members['removeAll']
                members['removeAll'] = _colRemoveAll
            if 'retainAll' in members :
                members['_retainAll'] = members['retainAll']
                members['retainAll'] = _colRetainAll

def _listGetItem(self, ndx) :
    if isinstance(ndx, slice) :
        start = ndx.start
        stop = ndx.stop
        if start < 0 :
            start = self.size() + start
        if stop < 0 :
            stop = self.size() + stop
        return self.subList(start, stop)
    else:
        if ndx < 0 :
            ndx = self.size() + ndx
        return self.get(ndx)
        
def _listSetItem(self, ndx, v) :
    if isinstance(ndx, slice) :
        start = ndx.start
        stop = ndx.stop
        if start < 0 :
            start = self.size() + start
        if stop < 0 :
            stop = self.size() + stop
        for i in range(start, stop) :
            self.remove(start)
        if operator.isSequenceType(v) :
            ndx = start
            for i in v :
                self.add(ndx, i)
                ndx += 1
    else:
        if ndx < 0 :
            ndx = self.size() + ndx
        self.set(ndx, v)

def _listAddAll(self, v, v2=None):
    if isPythonSequence(v) :
        r = False
        if v2 is not None : # assume form (int, values)
            for i in range(len(v2)) :
                r = r or self.add(v+i, v2[i])
                ndx +=1
        else:    
            for i in v :
                r = self.add(i) or r
        return r
    else:
        return self._addAll(v)

class ListCustomizer(object) :
    _METHODS = {
        '__setitem__' : _listSetItem,
        '__getitem__' : _listGetItem,
    }
    
    def canCustomize(self, name, jc) :
        if name == 'java.util.List' :
            return True
        return jc.isSubclass('java.util.List')
        
    def customize(self, name, jc, bases, members) :
        if name == 'java.util.List' :
            members.update(ListCustomizer._METHODS)
        else:
            if 'addAll' in members :
                members['_addAll'] = members['addAll']
                members['addAll'] = _listAddAll            
       
def isPythonMapping(v):       
    if operator.isMappingType(v) :
        if not hasattr(v.__class__, '__metaclass__') or v.__class__.__metaclass__ is _jclass._JavaClass :
            return True
    return False
        
def _mapLength(self) :
    return self.size()
    
def _mapIter(self) :
    return self.keySet().iterator()

def _mapDelItem(self, i) :
    return self.remove(i)

def _mapGetItem(self, ndx) :
    return self.get(ndx)
        
def _mapSetItem(self, ndx, v) :
    self.put(ndx, v)

def _mapPutAll(self, v):
    if isPythonMapping(v) :
        for i in v :
            self.put(i, v[i])
    else:        
        #do the regular method ...    
        self._putAll(v)
    
class MapCustomizer(object) :
    _METHODS = {
        '__len__' : _mapLength,
        '__iter__' : _mapIter,
        '__delitem__' : _mapDelItem,
        '__getitem__' : _mapGetItem,
        '__setitem__' : _mapSetItem,
    }
    
    def canCustomize(self, name, jc) :
        if name == 'java.util.Map' :
            return True
        return jc.isSubclass('java.util.Map')
        
    def customize(self, name, jc, bases, members) :
        if name == 'java.util.Map' :
            members.update(MapCustomizer._METHODS)
        else:
            if "putAll" in members :
                   members["_putAll"] = members["putAll"]
                   members["putAll"] = _mapPutAll

def _iterNext(self) :
    if self.hasNext() :
        return self._next()
    raise StopIteration
    
def _iterIter(self) :
    return self
    
class IteratorCustomizer(object) :
    _METHODS = {
        '__iter__' : _iterIter,
    }
    
    def canCustomize(self, name, jc) :
        if name == 'java.util.Iterator' :
            return True
        return jc.isSubclass('java.util.Iterator')
        
    def customize(self, name, jc, bases, members) :
        if name == 'java.util.Iterator' :
            members.update(IteratorCustomizer._METHODS)
        elif jc.isSubclass('java.util.Iterator') and 'next' in members:
            members['_next'] = members['next']
            members['next'] = _iterNext

def _enumNext(self) :
    if self.hasMoreElements() :
        return self.nextElement()
    raise StopIteration
    
def _enumIter(self) :
    return self
    

class EnumerationCustomizer(object) :
    _METHODS = {
        'next' : _enumNext,
        '__iter__' : _enumIter,
    }
    
    def canCustomize(self, name, jc) :
        return name == 'java.util.Enumeration'
        
    def customize(self, name, jc, bases, members) :
        members.update(EnumerationCustomizer._METHODS)
        



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
from jpype import *
import common
import unittest2
from sys import hexversion as ver

def _testMethod() :
    return 32

def _testMethod2() :
    return "Fooo!"

class C :
    def testMethod(self) :
        return 42
        
    def testMethod2(self) :
        return "Bar"

    def write(self, bytes, start, length) :
        print 'aaaaa'
        print bytes.__class__, bytes[0]
        print start
        print length

class ProxyTestCase(common.JPypeTestCase) :

    def setUp(self):
        super(ProxyTestCase, self).setUp()
        self.package = JPackage("jpype.proxy")

    def testProxyWithDict(self) :
        d = {
            'testMethod' : _testMethod,
            'testMethod2' : _testMethod2,
        }
        itf2 = self.package.ITestInterface3
        Test3 = self.package.Test3
        proxy = JProxy(itf2, dict=d)
    
        Test3.testProxy(proxy)

    def testProxyWithInst(self) :
        itf2 = self.package.ITestInterface3
        Test3 = self.package.Test3

        c = C()
        proxy = JProxy(itf2, inst=c)
        Test3.testProxy(proxy)   

    def testProxyWithThread(self) :
        itf2 = self.package.ITestInterface3
        Test3 = self.package.Test3

        c = C()
        proxy = JProxy(itf2, inst=c)

        t3 = Test3()
        t3.testProxyWithThread(proxy)

    @unittest2.skipIf(ver > 0x020703ff, 'broken, see ISSUE #67')
    def testProxyWithArguments(self) :
        itf2 = self.package.ITestInterface2
        Test3 = self.package.Test3

        c = C()
        proxy = JProxy(itf2, inst=c)
        Test3().testCallbackWithParameters(proxy)
    
    @unittest2.skipIf(ver > 0x020703ff, 'broken, see ISSUE #67')
    def testProxyWithMultipleInterface(self) :
        itf2 = self.package.ITestInterface2
        itf3 = self.package.ITestInterface3
        Test3 = self.package.Test3

        c = C()
        proxy = JProxy([itf2,itf3], inst=c)
        Test3().testCallbackWithParameters(proxy)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#	   http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
import _jclass, _jpype
import new

_CLASSES = {}

def JException(t) :
	return t.PYEXC

class JavaException(Exception) :
	def __init__(self, *data) :
		if isinstance(data[0], tuple) and data[0][0] is _jclass._SPECIAL_CONSTRUCTOR_KEY :
			# this is wrapped :
			self.__javaobject__ = data[0][1]
			self.__javaclass__ = self.__javaobject__.__class__
		else:
			self.__javaclass__ = _jclass.JClass(self.__class__.JAVACLASSNAME)
			self.__javaobject__ = self.__javaclass__(*data)
			
		Exception.__init__(self, self.__javaobject__)
		
	def javaClass(self) :
		return self.__javaclass__
		
	def message(self) :
		return self.__javaobject__.getMessage()
		
	def stacktrace(self) :
		StringWriter = _jclass.JClass("java.io.StringWriter")
		PrintWriter = _jclass.JClass("java.io.PrintWriter")
		sw = StringWriter()
		pw = PrintWriter(sw)
		
		self.__javaobject__.printStackTrace(pw)
		pw.flush()
		r = sw.toString()
		sw.close()
		return r

	def __str__(self) :
		return self.__javaobject__.toString()

def _initialize() :
	_jpype.setJavaExceptionClass(JavaException)
		
def _makePythonException(name, bc) :
	if _CLASSES.has_key(name) :
		return _CLASSES[name]
		
	if name == 'java.lang.Throwable' :
		bases = (JavaException,)
	else:
		bases = (_makePythonException(bc.getName(), bc.getBaseClass()) ,)
		
	ec = new.classobj(name+"PyRaisable", bases, {'JAVACLASSNAME' : name})
	
        _CLASSES[name] = ec
	return ec



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#	   http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
import operator
import sys
import _jpype
import _jclass
import _jwrapper

_CLASSES = {}
_CUSTOMIZERS = []

def _initialize() :
	_jpype.setJavaArrayClass(_JavaArrayClass)
	_jpype.setGetJavaArrayClassMethod(_getClassFor)
	
	registerArrayCustomizer(CharArrayCustomizer())
	registerArrayCustomizer(ByteArrayCustomizer())
	
def registerArrayCustomizer(c) :
	_CUSTOMIZERS.append(c)

class _JavaArrayClass(object) :
	def __init__(self, jo) :
		self.__javaobject__ = jo
		
	def __str__(self) :
		return str(tuple(self))
	
	def __len__(self) : 
		return _jpype.getArrayLength(self.__javaobject__)
		
	def __iter__(self) : 
		return _JavaArrayIter(self)
		
	def __getitem__(self, ndx ) : 
		return _jpype.getArrayItem(self.__javaobject__, ndx)
		
	def __setitem__(self, ndx, val) : 
		_jpype.setArrayItem(self.__javaobject__, ndx, val)
		
	def __getslice__(self, i, j) : 
		if j == sys.maxsize:
			j = _jpype.getArrayLength(self.__javaobject__)
		return _jpype.getArraySlice(self.__javaobject__, i, j)
		
	def __setslice__(self, i, j, v) : 
		if j == sys.maxsize:
			j = _jpype.getArrayLength(self.__javaobject__)
		_jpype.setArraySlice(self.__javaobject__, i, j, v)

def _jarrayInit(self, *args) :
	if len(args) == 2 and args[0] == _jclass._SPECIAL_CONSTRUCTOR_KEY :
		_JavaArrayClass.__init__(self, args[1])
	elif len(args) != 1 :
		raise ParameterException, "Array classes only take 2 parameters, %s given" % (len(args)+1,)
	else:
		values = None
		if operator.isSequenceType(args[0]) :
			sz = len(args[0])
			values = args[0]
		else :
			sz = args[0]
			
		_JavaArrayClass.__init__(self, _jpype.newArray(self.__class__.__javaclass__, sz))
		
		if values is not None :
			_jpype.setArraySlice(self.__javaobject__, 0, sz, values)
		
	
class _JavaArrayIter(object) :
	def __init__(self, a) :
		self._array = a
		self._ndx = -1
		
	def __iter__(self) :
		return self
		
	def next(self) :
		self._ndx += 1
		if self._ndx >= len(self._array) :
			raise StopIteration
		return self._array[self._ndx]
		
class _JavaArray(type) :
	pass

def _defineArrayClass(name, jt):
	members = {
		"__init__" : _jarrayInit,
		"__javaclass__" : jt,
	}
	
	bases = [_JavaArrayClass]
	
	for i in _CUSTOMIZERS :
		if i.canCustomize(name, jt) :
			i.customize(name, jt, bases, members)
	
	return _JavaArray(name, tuple(bases), members)
	
def _getClassFor(name) :
	if not _CLASSES.has_key(name) :
		jc = _jpype.findArrayClass(name)
		_CLASSES[name] = _defineArrayClass(name, jc)

	return _CLASSES[name]
	

def JArray(t, ndims=1) :
	if issubclass(t, _jwrapper._JWrapper) :
		t = t.typeName

	elif isinstance(t, _JavaArray) :
		t = t.typeName

	elif issubclass(t, _jclass._JAVAOBJECT):
		t = t.__name__
		
	elif not isinstance(t, str) and not isinstance(t, unicode) :
		raise TypeError, "Argument must be a java class, java array class, java wrapper or string representing a java class"
		
	arrayTypeName = t + ('[]'*ndims)
	
	return _getClassFor(arrayTypeName)
		
def _charArrayStr(self) :
	return ''.join(self)		
	
def _charArrayUnicode(self) :
	return u''.join(self)		

class CharArrayCustomizer(object) :
	def canCustomize(self, name, jc) :
		if name == 'char[]' :
			return True
		return False
		
	def customize(self, name, jc, bases, members) :
		members['__str__'] =  _charArrayStr   
		members['__unicode__'] =  _charArrayUnicode   
	
def _byteArrayStr(self) :
	s = _jclass.JClass('java.lang.String')(self)
	return s.toString()
	
	
class ByteArrayCustomizer(object) :
	def canCustomize(self, name, jc) :
		if name == 'byte[]' :
			return True
		return False
		
	def customize(self, name, jc, bases, members) :
		members['__str__'] =  _byteArrayStr   
	
		
	
		
	
	


#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
import sys
import _core
import _jproxy
import _jclass

def setupGuiEnvironment(cb):
	if sys.platform == 'darwin' :
		from PyObjCTools import AppHelper
		m = {'run' : cb}
		proxy = _jproxy.JProxy('java.lang.Runnable', m)
		cbthread = _jclass.JClass("java.lang.Thread")(proxy)
		cbthread.start()
		AppHelper.runConsoleEventLoop()
	else:
		cb()
		
def shutdownGuiEnvironment() :
	if sys.platform == 'darwin' :
		from PyObjCTools import AppHelper
		AppHelper.stopEventLoop()
	


		



import sys
from jpype import *

__JMenuBar = javax.swing.JMenuBar
__JMenu = javax.swing.JMenu

def buildMenuBar(menuDef) :
    mb = __JMenuBar()

    for i in menuDef :
        jm = buildMenu(i[0], i[1]) 
        mb.add( JObject(jm, __JMenu) )
        
    return mb
        
def buildMenu(name, menuDef) :
    jm = __JMenu(name)
    
    for i in menuDef :
        if i is None :
            jm.addSeparator()
        elif isinstance(i, list) or isinstance(i, tuple) :
            jm2 = buildMenu(i[0], i[1])
            jm.add(jm2)
        else:
            jm.add(i.proxy)
                 
    return jm


from jpype import *;

class AbstractAction(object) :
    ACCELERATOR_KEY    = javax.swing.Action.ACCELERATOR_KEY
    ACTION_COMMAND_KEY = javax.swing.Action.ACTION_COMMAND_KEY
    DEFAULT            = javax.swing.Action.DEFAULT
    LONG_DESCRIPTION   = javax.swing.Action.LONG_DESCRIPTION
    MNEMONIC_KEY       = javax.swing.Action.MNEMONIC_KEY
    NAME               = javax.swing.Action.NAME
    SHORT_DESCRIPTION  = javax.swing.Action.SHORT_DESCRIPTION
    SMALL_ICON         = javax.swing.Action.SMALL_ICON
    
    def __init__(self, cb, name=None, icon=None) :
        object.__init__(self)
        
        self.__proxy = JProxy(javax.swing.Action, inst=self)
        self.__values = {}
        self.__cb = cb
        self.__listeners = []
        self.__enabled = True
        
        if name is not None :
            self.putValue(AbstractAction.NAME, name)
            
        if icon is not None :
            self.putValue(AbstractAction.SMALL_ICON, icon)
    
    proxy = property(lambda self : self.__proxy) 
    
    def addPropertyChangeListener(self, listener) :
        self.__listeners.append(listener)
        
    def getValue(self, key) :
        return self.__values.get(key, None)
          
    def isEnabled(self) :
        if self.__enabled :
            return True
        return False
          
    def putValue(self, key, value) :
        oldVal = self.__values.get(key, None)
        if oldVal != value :
            self.__values[key] = value
            self.__notify(key, oldVal, value)
          
    def removePropertyChangeListener(self,listener) :
        self.__listeners.remove(listener)
          
    def setEnabled(self,b) : 
        if (b and not self.__enabled) or ( not b and self.__enabled) :
            self.__enabled = b
            self.__notify("enabled", java.lang.Boolean(not b), java.lang.Boolean(b))  

    def actionPerformed(self, ev) :
        self.__cb(ev) 

    def __notify(self, k, oldVal, newVal) :
        ev = java.beans.PropertyChangeEvent(self.__proxy, k, oldVal, newVal)
        for i in self.__listeners :
            i.propertyChange(ev) 



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#	   http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
import jpype
from jpype import JString, java, JArray
import common
import time

class AttributeTestCase(common.JPypeTestCase) :
	def setUp(self) :
		common.JPypeTestCase.setUp(self)
		self.__jp = self.jpype.attr
		
	def testWithBufferStrategy(self):
		j = jpype.JPackage("jpype").attr.ClassWithBuffer
		print j
		
	def testCallOverloadedMethodWithCovariance(self):
		# This is a JDk5-specific problem.
		
		h = jpype.java.lang.StringBuffer()
		h.delete(0, 0)
		
 	def testCallStaticString(self) :
 		h = self.__jp.Test1()	
 		v = h.testString(JString("abcd"), JString("abcde"))
 		
 		assert v[0] == 'abcd'
 		assert v[1] == 'abcde'
 		
 	def testCallStaticUnicodeString(self) :
 		h = self.__jp.Test1()	
 		v = h.testString(JString(u"abcd"), JString(u"abcde"))
 		
 		assert v[0] == 'abcd'
 		assert v[1] == 'abcde'
 
 	def testCallString(self) :
 		v = self.__jp.Test1.testStaticString("a", "b")
 		assert v[0] == 'a'
 		assert v[1] == 'b'
 
 	def testCallUnicodeString(self) :
 		v = self.__jp.Test1.testStaticString(u"a", u"b")
 		assert v[0] == 'a'
 		assert v[1] == 'b'
 
 	def testCallStringWithNone(self) :
 		v = self.__jp.Test1.testStaticString("a", None)
 		assert v[0] == 'a'
 		assert v[1] == None
 
 	def testWithHolder(self) :
 		print 'testWithHolder'
 		holder = self.__jp.Holder()
 		holder.f = "ffff"
 		assert holder.f == "ffff"
 		self.__jp.Test1.testStaticHolder(holder)
 	
 	def testWithSubHolder(self) :
 		h2 = self.__jp.SubHolder()
 		h2.f = "subholder"
 		self.__jp.Test1.testStaticHolder(h2)
 	
 	def testCallWithArray(self) :
 		h2 = self.__jp.Test1()
 		StringArray = JArray(JString)
 		v = StringArray(["Foo", "bar"])
 		t = self.__jp.Test1()
 		t.testStringArray(v)
 
 	def testGetStaticValue(self) :
 		assert str(self.__jp.Test1.objectValue) == "234"
 		
 	def testGetStaticByInstance(self) :
 		h = self.__jp.Test1()
 		assert str(h.objectValue) == "234"
 		
 	def testGetNonStatic(self) :
 		h = self.__jp.Test1()
 		assert h.stringValue == "Foo"
 
 	def testSetStaticValue(self) :
 		self.__jp.Test1.objectValue = java.lang.Integer(43)
 		assert str(self.__jp.Test1.objectValue) == "43"
 		self.__jp.Test1.reset()
 
 	def testSetNonStaticValue(self) :
 		h = self.__jp.Test1()
 		h.stringValue="bar"
 		assert h.stringValue == "bar"
 
 	def testReturnSubClass(self) :
 		h = self.__jp.Test1()
 		v = h.getSubClass()
 		assert isinstance(v, self.__jp.SubHolder)
 		
 	def testCallWithClass(self) :
 		h = self.__jp.Test1()
 		h.callWithClass(java.lang.Comparable)
 		
 	def testCallSuperclassMethod(self) :
 		h = self.__jp.Test2()
 		h.test2Method()
 		h.test1Method()		
 		
 	def testCallWithLong(self) :
 		h = self.__jp.Test1()
 		l = long(123)
 		
 		h.setByte(l)
 		h.setShort(l)
 		h.setInt(l)
 		
 	def testCharAttribute(self) :
 		h = self.__jp.Test1()
 		h.charValue = u'b'
 		
 		print h.charValue
 		print repr(h.charValue)
 		
 		assert h.charValue == u'b'
 		
 	def testGetPrimitiveType(self) :
 		Integer = jpype.JClass("java.lang.Integer")
 		intType = Integer.TYPE
 		
 	def testDifferentiateClassAndObject(self) :
 		h = self.__jp.Test1()
 		
 		assert h.callWithSomething(self.__jp.Test1) == u"Class"
 		assert h.callWithSomething(jpype.JObject(self.__jp.Test1, jpype.java.lang.Object)) == u"Object"

 	def testToString(self):
 		h = self.__jp.Test1()
 		assert str(h) == 'aaa'
 
 	def testSuperToString(self):
 		h = self.__jp.Test2()
 		print h
 		assert str(h) == 'aaa'
 		del h
		
#	def testStringToConversion(self):
#		try :
#			jpype.ConversionConfig.string = False
#			for i in range(1) :
#				h = self.__jp.Test1()
#				
#				start = time.time();
#				for j in range(10):
#					ts = h.getBigString()
#				stop = time.time()
#				print ts.__class__, (stop-start), 'ms'
#				
#				# for comparison ... convert a string to JStrng and back
#				s = "".join([" "]*1024*1024*5)
#				start = time.time()
#				js = JString(s)
#				stop = time.time()
#				print "converted to JString in", (stop-start), 'ms', len(s)
#				
#				start = time.time()
#				cs = str(JString(s))
#				print cs
#				print "converted to String in", (stop-start), 'ms', len(cs), cs
#		finally :
#			jpype.ConversionConfig.string = True
		
 	def testComplexMethodOvlerloading(self) :
 		c = self.__jp.TestOverloadC()
 		print c.foo(1)
 		assert c.foo(1) == "foo(int) in C: 1"
 		assert c.foo() == "foo() in A"



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
#*****************************************************************************
import _jpype
import _jexception
import _jcollection
from _pykeywords import KEYWORDS


_CLASSES = {}

_SPECIAL_CONSTRUCTOR_KEY = "This is the special constructor key"

_JAVAOBJECT = None
_JAVATHROWABLE = None
_COMPARABLE = None
_RUNTIMEEXCEPTION = None

_CUSTOMIZERS = []

_COMPARABLE_METHODS = {
        "__cmp__": lambda self, o: self.compareTo(o)
        }


def _initialize():
    global _COMPARABLE, _JAVAOBJECT, _JAVATHROWABLE, _RUNTIMEEXCEPTION

    import _jpackage
    _JAVAOBJECT = JClass("java.lang.Object")
    _JAVATHROWABLE = JClass("java.lang.Throwable")
    _RUNTIMEEXCEPTION = JClass("java.lang.RuntimeException")
    _jpype.setJavaLangObjectClass(_JAVAOBJECT)
    _jpype.setGetClassMethod(_getClassFor)
    _jpype.setSpecialConstructorKey(_SPECIAL_CONSTRUCTOR_KEY)


def registerClassCustomizer(c):
    _CUSTOMIZERS.append(c)


def JClass(name):
    jc = _jpype.findClass(name)
    if jc is None:
        raise _RUNTIMEEXCEPTION.PYEXC("Class %s not found" % name)

    return _getClassFor(jc)


def _getClassFor(javaClass):
    name = javaClass.getName()
    if name in _CLASSES:
        return _CLASSES[name]

    pyJavaClass = _JavaClass(javaClass)
    _CLASSES[name] = pyJavaClass
    return pyJavaClass


def _javaNew(self, *args):
    return object.__new__(self)


def _javaExceptionNew(self, *args):
    return Exception.__new__(self)


def _javaInit(self, *args):
    object.__init__(self)

    if len(args) == 1 and isinstance(args[0], tuple) and args[0][0] is _SPECIAL_CONSTRUCTOR_KEY:
        self.__javaobject__ = args[0][1]
    else:
        self.__javaobject__ = self.__class__.__javaclass__.newClassInstance(*args)


def _javaGetAttr(self, name):
    try:
        r = object.__getattribute__(self, name)
    except AttributeError, ex:
        if name in dir(self.__class__.__metaclass__):
            r = object.__getattribute__(self.__class__, name)
        else:
            raise ex

    if isinstance(r, _jpype._JavaMethod):
        return _jpype._JavaBoundMethod(r, self)
    return r


class _JavaClass(type):
    def __new__(mcs, jc):
        bases = []
        name = jc.getName()

        static_fields = {}
        constants = []
        members = {
                "__javaclass__": jc,
                "__init__": _javaInit,
                "__str__": lambda self: self.toString(),
                "__hash__": lambda self: self.hashCode(),
                "__eq__": lambda self, o: self.equals(o),
                "__ne__": lambda self, o: not self.equals(o),
                "__getattribute__": _javaGetAttr,
                }

        if name == 'java.lang.Object' or jc.isPrimitive():
            bases.append(object)
        elif not jc.isInterface():
            bjc = jc.getBaseClass(jc)
            bases.append(_getClassFor(bjc))

        if _JAVATHROWABLE is not None and jc.isSubclass("java.lang.Throwable"):
            members["PYEXC"] = _jexception._makePythonException(name, bjc)

        itf = jc.getBaseInterfaces()
        for ic in itf:
            bases.append(_getClassFor(ic))

        if len(bases) == 0:
            bases.append(_JAVAOBJECT)

        # add the fields
        fields = jc.getClassFields()
        for i in fields:
            fname = i.getName()
            if fname in KEYWORDS:
                fname = fname + "_"

            if i.isStatic():
                g = lambda self, fld=i: fld.getStaticAttribute()
                s = None
                if not i.isFinal():
                    s = lambda self, v, fld=i: fld.setStaticAttribute(v)
                static_fields[fname] = property(g, s)
            else:
                g = lambda self, fld=i: fld.getInstanceAttribute(self.__javaobject__)
                s = None
                if not i.isFinal():
                    s = lambda self, v, fld=i: fld.setInstanceAttribute(self.__javaobject__, v)
                members[fname] = property(g, s)

        # methods
        methods = jc.getClassMethods()  # Return tuple of tuple (name, method).
        for jm in methods:
            mname = jm.getName()
            if mname in KEYWORDS:
                mname = mname + "_"

            members[mname] = jm

        for i in _CUSTOMIZERS:
            if i.canCustomize(name, jc):
                i.customize(name, jc, bases, members)

        # remove multiple bases that would cause a MRO problem
        toRemove = set()
        for c in bases:
            for d in bases:
                if c == d:
                    continue
                if issubclass(c, d):
                    toRemove.add(d)

        for i in toRemove:
            bases.remove(i)

        # Prepare the meta-metaclass
        meta_bases = []
        for i in bases:
            if i is object:
                meta_bases.append(mcs)
            else:
                meta_bases.append(i.__metaclass__)

        metaclass = type.__new__(type, name + "$$Static", tuple(meta_bases),
                                 static_fields)
        members['__metaclass__'] = metaclass
        result = type.__new__(metaclass, name, tuple(bases), members)

        return result



from os import path
import shutil
import tempfile
from jpype import *

lucene_jar = path.join("..", "build", "lucene-1.4.3.jar")
if not path.isfile(lucene_jar):
    raise IOError, "Please provide %s" % path.abspath(lucene_jar)
startJVM(getDefaultJVMPath(),'-Djava.class.path=%s' % lucene_jar) 

QueryParser = JClass("org.apache.lucene.queryParser.QueryParser") 
IndexSearcher = JClass("org.apache.lucene.search.IndexSearcher") 
IndexReader = JClass("org.apache.lucene.index.IndexReader") 
StandardAnalyzer = JClass("org.apache.lucene.analysis.standard.StandardAnalyzer") 
FSDirectory = JClass("org.apache.lucene.store.FSDirectory") 
IndexWriter = JClass("org.apache.lucene.index.IndexWriter")
SimpleAnalyzer = JClass("org.apache.lucene.analysis.SimpleAnalyzer")

tmppath = tempfile.mkdtemp()
IndexWriter(tmppath, SimpleAnalyzer(), True).close() 
 
directory = FSDirectory.getDirectory(tmppath, False) 
reader = IndexReader.open(directory) 
searcher = IndexSearcher(reader) 
queryparser = QueryParser.parse("wenger","contents",StandardAnalyzer()) 
print queryparser.rewrite
print queryparser.rewrite.matchReport(reader)
qp = queryparser.rewrite(reader) 
print qp
print searcher.search.matchReport(qp) 
hits = searcher.search(qp) 

shutil.rmtree(tmppath)



#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
from _jpackage import *
from _jclass import *
from _jarray import *
from _jwrapper import *
from _jproxy import *
from _jexception import *
from _core import *
from _gui import *
import reflect
import nio


import JClassUtil
    
def JIterator(it) :
	"""Deprecated"""
	return it
        
java = JPackage("java")
javax = JPackage("javax")
    
    


#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#	   http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************
import _jclass

def _initialize() :
	_jclass.registerClassCustomizer(ComparableCustomizer())
	
class ComparableCustomizer(object) :
	_METHODS = { 
		"__cmp__" : lambda self, o : self.compareTo(o)
	}
	
	def canCustomize(self, name, jc) :
		return name == 'java.lang.Comparable'
		
	def customize(self, name, jc, bases, members) :
		members.update(ComparableCustomizer._METHODS)


#*****************************************************************************
#   Copyright 2004-2008 Steve Menard
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#   
#*****************************************************************************

import _jpype
import _jclass

class JPackage(object) :
    def __init__(self, name) :
        self.__name = name
        
    def __getattribute__(self, n) :
        try :
            return object.__getattribute__(self, n)
        except :
            # not found ...
            
            # perhaps it is a class?
            subname = "%s.%s" % (self.__name, n)
            cc = _jpype.findClass(subname)
            if cc is None :
                # can only assume it is a sub-package then ...
                cc = JPackage(subname)
            else:
                cc = _jclass._getClassFor(cc)
                
            self.__setattr__(n, cc, True)
            
            return cc
            
    def __setattr__(self, n, v, intern=False) :
        if not n[:len("_JPackage")] == '_JPackage' and not intern : # NOTE this shadows name mangling
            raise RuntimeError, "Cannot set attributes in a package"+n
        object.__setattr__(self, n, v)
        
    def __str__(self) :
        return "<Java package %s>" % self.__name
        
    def __call__(self, *arg, **kwarg) :
        raise TypeError, "Package "+self.__name+" is not Callable"
